import {attributeChangeEventPrefix} from "../Constants";

/**
 * @function componentEventname
 * 
 * creates an event name for an component
 * 
 * @param {string} eventType 
 * @param {string|HTMLElement|Component} node 
 * @param {string} [separator] default is ":"
 * 
 * @returns {string} 
 */
export const componentEventname = (eventType, node, separator = ":" ) => {	
	let nodename = "unsupported";
	if(typeof node === "string")
		nodename = node;
	else if(node instanceof HTMLElement)
		nodename = node.nodeName;
	else if(typeof node.NODENAME === "string")
		nodename = node.NODENAME;
	else throw new Error(`${typeof node} is not supported as parameter "node"!`);
	
   return `${nodename.toLowerCase()}${separator}${eventType}`;//use @ as separtor and not :
};

/**
 * @function attributeChangeEventname
 *  * 
 * @param {string} attribute 
 * @param {string|HTMLElement|Component} node 
 * @param {string} [separator] default is ":"
 * 
 * @returns {string}
 */
export const attributeChangeEventname = (attribute, node, separator = ":"  ) => {
    return componentEventname(`${attributeChangeEventPrefix}-${attribute}`, node, separator);
};

export default {componentEventname, attributeChangeEventname}